# $Id$

import os
from apelib.fs.annotated import AnnotatedFilesystem, remainder_ext, properties_ext, remainder_ann, section_re

DEBUG = os.environ.get('CAESAR_DEBUG')

class PropertyFilesystem(AnnotatedFilesystem):

    propprefix = 'caesar:'

    def get_annotations(self, path):
        if DEBUG: print "got get_annotations for path '%s'" % path
        res = self._anns_cache.get(path)
        if res is not None:
            return res
        res = {}
        data = self.ops.readproperty(path, self.propprefix + remainder_ext)
        if DEBUG: print "read remainder '%s'" % data
        if ( data != None ):
            # We have a remainder
            res[remainder_ann] = data
        data = self.ops.readproperty(path, self.propprefix + properties_ext)
        if DEBUG: print "read properties '%s'" % data
        if ( data == None ):
            # caesar svn property for "properties" not set
            self._anns_cache.set(path, res)
            return res
        pos = 0
        prev_section_name = None
        while 1:
            match = section_re.search(data, pos)
            if match is None:
                endpos = len(data)
            else:
                endpos = match.start()
            if prev_section_name is not None:
                # get the data and decode.
                section = data[pos:endpos].replace('[[', '[')
                res[prev_section_name] = section
            if match is None:
                break
            else:
                prev_section_name = match.group(1)
                pos = match.end()
        self._anns_cache.set(path, res)
        return res

    def write_annotations(self, path, anns):
        props_data = ''
        rem_data = ''
        items = anns.items()
        items.sort()
        for name, value in items:
            props_data += self._format_section(name, value)
        self._write_or_remove(path, self.propprefix + properties_ext, props_data)
        self._write_or_remove(path, self.propprefix + remainder_ext, rem_data)
        self._anns_cache.invalidate(path)
        self._dir_cache.invalidate(self.ops.dirname(path))

    def _write_or_remove(self, path, name, data):
        ops = self.ops
        if data:
            ops.writeproperty(path, name, data)
        else:
            ops.writeproperty(path, name, None)
